require 'emcee/cache'

class Emcee
  class Resource
    class << self
      def search(emcee, type)
        convert_results_list(run_ral_action(emcee, 'search', {:type => type}))
      end

      def find(emcee, type, title, include_managed_absent=nil)
        convert_results(run_ral_action(emcee, 'find', { :type => type, :title => title }), include_managed_absent)
      end

      def create(emcee, resource)
        # To get around stupid Sinatra hash default proc automagic so Marshal works
        resource["parameters"] = {}.merge(resource['parameters'])
        run_ral_action(emcee, 'create', resource)
      end

      def run_ral_action(emcee, action_name, args)
        puppetral_action(emcee, action_name).run(args)
      end

      def puppetral_action(emcee, action_name)
        emcee.agents_by_name("puppetral").first.action_by_name(action_name)
      end

      def convert_results_list(list)
        out = list.collect { |results|
          results.contents.collect {|k,v| convert_search_result(v, results.node_name) }
        }.flatten
        omit_absent_search_results(out)
      end

      def convert_search_result(result, node_name)
        result.delete('tags')
        result.delete('exported')
        result.merge('node_name' => node_name)
      end

      # XXX : This is technical debt. The code paths for
      # omit_absent_search_results and omit_absent_results should be
      # consolidated.
      def omit_absent_search_results(results)
        undesirables = [:absent, :purged]
        results.reject { |resource| !resource.has_key?('parameters') || undesirables.include?(resource['parameters'][:ensure]) }
      end

      def convert_results(results, include_managed_absent=nil)
        omit_absent_results(results, include_managed_absent).map {|result| convert_result(result) }
      end

      def omit_absent_results(results, include_managed_absent=nil)
        undesirables = [:absent, :purged]
        good_results, errored_results = results.partition { |result| result.contents['parameters'] }

        errored_results.each do |result|
          puts "Agent node #{result.node_name} had exception while processing request: #{result.status_message}"
        end

        if include_managed_absent
          good_results.reject {|result| (result.contents[:managed] != true && undesirables.include?(result.contents['parameters'][:ensure]))}
        else
          good_results.reject {|result| undesirables.include?(result.contents['parameters'][:ensure])}
        end
      end

      def convert_result(result)
        {
          'node_name'  => result.node_name,
          'type'       => result.contents['type'],
          'title'      => result.contents['title'],
          'managed'    => result.contents[:managed],
          'parameters' => result.contents['parameters'],
        }
      end
    end
  end
end
